<?php
/**
 * Product Visibility by Country for WooCommerce - Core Class
 *
 * @version 1.1.5
 * @since   1.0.0
 * @author  Algoritmika Ltd.
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_PVBC_Core' ) ) :

class Alg_WC_PVBC_Core {

	/**
	 * Constructor.
	 *
	 * @version 1.1.5
	 * @since   1.0.0
	 * @todo    [dev] back-end: product terms: add fields to "add category/tag" pages also (i.e. not only to "edit" pages)
	 * @todo    [feature] hide description
	 * @todo    [feature] add quick and bulk edit options
	 * @todo    [feature] add option to set visibility by categories / tags
	 * @todo    [feature] add option to hide menu items
	 * @todo    [feature] add option to set visibility for variations
	 */
	function __construct() {
		$this->is_wc_version_below_3 = version_compare( get_option( 'woocommerce_version', null ), '3.0.0', '<' );
		if ( 'yes' === get_option( 'alg_wc_pvbc_enabled', 'yes' ) ) {
			// Core
			if ( ! is_admin() || ( defined( 'DOING_AJAX' ) && DOING_AJAX ) ) {
				// Front-end: Products
				if ( 'yes' === get_option( 'alg_wc_pvbc_visibility', 'yes' ) ) {
					add_filter( 'woocommerce_product_is_visible',               array( $this, 'product_by_country_visibility' ), PHP_INT_MAX, 2 );
				}
				if ( 'yes' === get_option( 'alg_wc_pvbc_purchasable', 'no' ) ) {
					add_filter( 'woocommerce_is_purchasable',                   array( $this, 'product_by_country_purchasable' ), PHP_INT_MAX, 2 );
				}
				if ( 'yes' === get_option( 'alg_wc_pvbc_query', 'no' ) ) {
					add_action( 'pre_get_posts',                                array( $this, 'product_by_country_pre_get_posts' ) );
					if ( 'yes' === get_option( 'alg_wc_pvbc_query_widget', 'no' ) ) {
						add_filter( 'woocommerce_products_widget_query_args',   array( $this, 'product_by_country_widget_query' ), PHP_INT_MAX );
					}
					// Redirect
					$this->page_to_redirect = get_option( 'alg_wc_pvbc_query_redirect_url', '' );
					if ( ! empty( $this->page_to_redirect ) ) {
						add_action( 'template_redirect',                        array( $this, 'redirect_404' ) );
					}
				} else {
					// Price
					if ( 'yes' === get_option( 'alg_wc_pvbc_hide_price', 'no' ) ) {
						add_filter( 'woocommerce_get_price_html',               array( $this, 'product_by_country_price_html' ), PHP_INT_MAX, 2 );
						add_filter( 'woocommerce_product_get_price',            array( $this, 'product_by_country_price' ),      PHP_INT_MAX, 2 );
					}
					// Info
					if ( 'yes' === get_option( 'alg_wc_pvbc_info_on_single_product', 'no' ) ) {
						add_action( 'woocommerce_single_product_summary',       array( $this, 'add_message_on_single_product' ), 31 );
					}
					if ( 'yes' === get_option( 'alg_wc_pvbc_info_on_loop', 'no' ) ) {
						add_action( 'woocommerce_after_shop_loop_item',         array( $this, 'add_message_on_loop' ), 11 );
					}
					add_shortcode( 'alg_wc_pvbc_translate',                     array( $this, 'language_shortcode' ) );
				}
				// Front-end: Product terms
				if ( 'yes' === apply_filters( 'alg_wc_pvbc', 'no', 'product_terms_enabled' ) ) {
					add_filter( 'get_terms',                                    array( $this, 'product_by_country_product_terms' ), PHP_INT_MAX, 4 );
				}
			} else {
				// Back-end: Admin products list
				if ( 'yes' === get_option( 'alg_wc_pvbc_add_column_visible_countries', 'no' ) ) {
					add_filter( 'manage_edit-product_columns',                  array( $this, 'add_product_columns' ),   PHP_INT_MAX );
					add_action( 'manage_product_posts_custom_column',           array( $this, 'render_product_column' ), PHP_INT_MAX );
				}
				// Back-end: Product terms
				if ( 'yes' === apply_filters( 'alg_wc_pvbc', 'no', 'product_terms_enabled' ) ) {
					add_action( 'product_cat_edit_form_fields',                 array( $this, 'product_terms_add_fields' ),  PHP_INT_MAX );
					add_action( 'product_tag_edit_form_fields',                 array( $this, 'product_terms_add_fields' ),  PHP_INT_MAX );
					add_action( 'edit_product_cat',                             array( $this, 'product_terms_save_fields' ), PHP_INT_MAX );
					add_action( 'edit_product_tag',                             array( $this, 'product_terms_save_fields' ), PHP_INT_MAX );
				}
			}
		}
	}

	/**
	 * product_by_country_price_html.
	 *
	 * @version 1.1.5
	 * @since   1.1.5
	 */
	function product_by_country_price_html( $price_html, $product ) {
		return ( ! $this->is_product_visible( $this->get_product_id_or_variation_parent_id( $product ), $this->get_country_by_ip() ) ?
			( '' != ( $content = get_option( 'alg_wc_pvbc_hide_price_content', '' ) ) ? do_shortcode( $content ) : '' ) : $price_html );
	}

	/**
	 * product_by_country_price.
	 *
	 * @version 1.1.5
	 * @since   1.1.5
	 */
	function product_by_country_price( $price, $product ) {
		return ( ! $this->is_product_visible( $this->get_product_id_or_variation_parent_id( $product ), $this->get_country_by_ip() ) ?
			'' : $price );
	}

	/**
	 * language_shortcode.
	 *
	 * @version 1.1.4
	 * @since   1.1.4
	 */
	function language_shortcode( $atts, $content = '' ) {
		// E.g.: `[alg_wc_pvbc_translate lang="DE" lang_text="Das Produkt ist in Ihrem Land nicht verfügbar." not_lang_text="The product is not available in your country."]`
		if ( isset( $atts['lang_text'] ) && isset( $atts['not_lang_text'] ) && ! empty( $atts['lang'] ) ) {
			return ( ! defined( 'ICL_LANGUAGE_CODE' ) || ! in_array( strtolower( ICL_LANGUAGE_CODE ), array_map( 'trim', explode( ',', strtolower( $atts['lang'] ) ) ) ) ) ?
				$atts['not_lang_text'] : $atts['lang_text'];
		}
		// E.g.: `[alg_wc_pvbc_translate lang="DE"]Das Produkt ist in Ihrem Land nicht verfügbar.[/alg_wc_pvbc_translate][alg_wc_pvbc_translate not_lang="DE"]The product is not available in your country.[/alg_wc_pvbc_translate]`
		return (
			( ! empty( $atts['lang'] )     && ( ! defined( 'ICL_LANGUAGE_CODE' ) || ! in_array( strtolower( ICL_LANGUAGE_CODE ), array_map( 'trim', explode( ',', strtolower( $atts['lang'] ) ) ) ) ) ) ||
			( ! empty( $atts['not_lang'] ) &&     defined( 'ICL_LANGUAGE_CODE' ) &&   in_array( strtolower( ICL_LANGUAGE_CODE ), array_map( 'trim', explode( ',', strtolower( $atts['not_lang'] ) ) ) ) )
		) ? '' : $content;
	}

	/**
	 * output_message.
	 *
	 * @version 1.1.4
	 * @since   1.1.4
	 * @todo    [feature] terms?
	 * @todo    [feature] position & priority
	 * @todo    [feature] replaced values (and/or shortcodes?), e.g. `%customer_country%`, `%product_title%`
	 */
	function output_message( $view ) {
		if ( ! $this->is_product_visible( get_the_ID(), $this->get_country_by_ip() ) ) {
			echo do_shortcode( get_option( 'alg_wc_pvbc_info_on_' . $view . '_content',
				'<p><strong>' . __( 'The product is not available in your country.', 'product-visibility-by-country-for-woocommerce' ) . '</strong></p>' ) );
		}
	}

	/**
	 * add_message_on_loop.
	 *
	 * @version 1.1.4
	 * @since   1.1.4
	 */
	function add_message_on_loop() {
		$this->output_message( 'loop' );
	}

	/**
	 * add_message_on_single_product.
	 *
	 * @version 1.1.4
	 * @since   1.1.4
	 */
	function add_message_on_single_product() {
		$this->output_message( 'single_product' );
	}

	/**
	 * redirect_404.
	 *
	 * @version 1.1.3
	 * @since   1.1.3
	 * @todo    [dev] (maybe) add `$wp_query->is_alg_wc_pvbc_hidden`
	 * @todo    [dev] (maybe) add `$wp_query->is_alg_wc_pvbc_hidden_product_id`
	 * @todo    [feature] add option to set redirect URL **per product**
	 */
	function redirect_404() {
		apply_filters( 'alg_wc_pvbc', true, 'redirect_404', array( 'core' => $this ) );
	}

	/**
	 * product_terms_add_fields.
	 *
	 * @version 1.1.1
	 * @since   1.1.1
	 */
	function product_terms_add_fields( $term ) {
		$html = '';
		$is_chosen_select = ( 'chosen_select' === get_option( 'alg_wc_pvbc_select_box_type', 'chosen_select' ) );
		$options = array(
			'visible' => array(
				'title' => __( 'Visible in countries', 'product-visibility-by-country-for-woocommerce' ),
				'desc'  => __( 'Visible in selected countries only.', 'product-visibility-by-country-for-woocommerce' ) . ' ' .
					__( 'If no countries are selected - term will be visible in all countries.', 'product-visibility-by-country-for-woocommerce' ) .
					( $is_chosen_select ? '' : '<br>' . __( 'Hold Control (Ctrl) key to select/deselect multiple countries. Hold Control + A to select all countries.', 'product-visibility-by-country-for-woocommerce' ) ),
			),
			'invisible' => array(
				'title' => __( 'Invisible in countries', 'product-visibility-by-country-for-woocommerce' ),
				'desc'  => __( 'Hidden in selected countries.', 'product-visibility-by-country-for-woocommerce' ) . ' ' .
					__( 'If no countries are selected - term will be visible in all countries.', 'product-visibility-by-country-for-woocommerce' ) .
					( $is_chosen_select ? '' : '<br>' . __( 'Hold Control (Ctrl) key to select/deselect multiple countries. Hold Control + A to select all countries.', 'product-visibility-by-country-for-woocommerce' ) ),
			),
		);
		foreach ( $options as $option_id => $option_data ) {
			$value = get_term_meta( $_GET['tag_ID'], '_alg_wc_pvbc_' . $option_id, true );
			if ( empty( $value ) ) {
				$value = array();
			}
			$select_options = '';
			foreach( alg_wc_pvbc()->metaboxes->get_countries() as $country_code => $country_title ) {
				$select_options .= '<option value="' . $country_code . '" ' . selected( in_array( $country_code, $value ), true, false ) . '>' . $country_title . '</option>';
			}
			$html .= '<tr class="form-field">' .
				'<th scope="row" valign="top"><label for="alg_wc_pvbc_' . $option_id . '">' . $option_data['title'] . '</label></th>' .
				'<td>' . '<select multiple name="alg_wc_pvbc_' . $option_id . '[]" id="alg_wc_pvbc_' . $option_id . '" style="width:95%;' .
					( $is_chosen_select ? '' : 'height:300px;' ) . '" class="' . ( $is_chosen_select ? 'chosen_select' : '' ) . '">' .
						$select_options . '</select>' . '<span class="description">' . $option_data['desc'] . '</span>' .
					( 'invisible' === $option_id ? '<input type="hidden" name="alg_wc_pvbc_edit_terms" value="1">' : '' ) .
				'</td>' .
			'</tr>';
		}
		echo $html;
	}

	/**
	 * product_terms_save_fields.
	 *
	 * @version 1.1.1
	 * @since   1.1.1
	 */
	function product_terms_save_fields() {
		if ( isset( $_POST['alg_wc_pvbc_edit_terms'] ) && isset( $_POST['tag_ID'] ) ) {
			$visible   = ( isset( $_POST['alg_wc_pvbc_visible'] )   ? $_POST['alg_wc_pvbc_visible']   : array() );
			$invisible = ( isset( $_POST['alg_wc_pvbc_invisible'] ) ? $_POST['alg_wc_pvbc_invisible'] : array() );
			update_term_meta( $_POST['tag_ID'], '_alg_wc_pvbc_visible',   $visible );
			update_term_meta( $_POST['tag_ID'], '_alg_wc_pvbc_invisible', $invisible );
		}
	}

	/**
	 * product_by_country_product_terms.
	 *
	 * @version 1.1.1
	 * @since   1.1.1
	 */
	function product_by_country_product_terms( $terms, $taxonomy, $query_vars, $term_query ) {
		return apply_filters( 'alg_wc_pvbc', $terms, 'product_terms', array( 'core' => $this, 'taxonomy' => $taxonomy ) );
	}

	/**
	 * add_product_columns.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function add_product_columns( $columns ) {
		$columns[ 'alg_wc_pvbc_countries' ] = __( 'Countries', 'product-visibility-by-country-for-woocommerce' );
		return $columns;
	}

	/**
	 * render_product_column.
	 *
	 * @version 1.1.1
	 * @since   1.0.0
	 * @todo    [dev] (maybe) add flags
	 */
	function render_product_column( $column ) {
		if ( 'alg_wc_pvbc_countries' === $column ) {
			$html = '';
			if ( $countries = get_post_meta( get_the_ID(), '_' . 'alg_wc_pvbc_visible', true ) ) {
				if ( is_array( $countries ) && ! empty( $countries ) ) {
					$html .= '<span style="color:green;">' . implode( ', ', $countries ) . '</span>';
				}
			}
			if ( $countries = get_post_meta( get_the_ID(), '_' . 'alg_wc_pvbc_invisible', true ) ) {
				if ( is_array( $countries ) && ! empty( $countries ) ) {
					if ( ! empty ( $html ) ) {
						$html .= '<br>';
					}
					$html .= '<span style="color:red;">' . implode( ', ', $countries ) . '</span>';
				}
			}
			echo $html;
		}
	}

	/**
	 * maybe_add_eu_countries.
	 *
	 * @version 1.1.2
	 * @since   1.1.2
	 */
	function maybe_add_eu_countries( $countries ) {
		return ( ! empty( $countries ) && is_array( $countries ) && in_array( 'EU', $countries ) ?
			array_merge( $countries, array( 'AT', 'BE', 'BG', 'CY', 'CZ', 'DE', 'DK', 'EE', 'ES', 'FI', 'FR', 'GB', 'GR', 'HU', 'HR', 'IE', 'IT', 'LT', 'LU', 'LV',
				'MT', 'NL', 'PL', 'PT', 'RO', 'SE', 'SI', 'SK' ) ) :
			$countries
		);
	}

	/**
	 * is_product_visible.
	 *
	 * @version 1.1.2
	 * @since   1.1.0
	 * @todo    [dev] (maybe) optimize by saving the result in `$this->saved_product_visibility[ $product_id ][ $country ]`
	 */
	function is_product_visible( $product_id, $country ) {
		$countries = $this->maybe_add_eu_countries( get_post_meta( $product_id, '_' . 'alg_wc_pvbc_visible',   true ) );
		if ( ! empty( $countries ) && is_array( $countries ) && ! in_array( $country, $countries ) ) {
			return false;
		}
		$countries = $this->maybe_add_eu_countries( get_post_meta( $product_id, '_' . 'alg_wc_pvbc_invisible', true ) );
		if ( ! empty( $countries ) && is_array( $countries ) &&   in_array( $country, $countries ) ) {
			return false;
		}
		return true;
	}

	/**
	 * product_by_country_widget_query.
	 *
	 * @version 1.1.0
	 * @since   1.1.0
	 * @todo    [dev] check `$args = $query_args;`
	 */
	function product_by_country_widget_query( $query_args ) {
		remove_action( 'pre_get_posts', array( $this, 'product_by_country_pre_get_posts' ) );
		$country                = $this->get_country_by_ip();
		$post__not_in           = ( isset( $query_args['post__not_in'] ) ? $query_args['post__not_in'] : array() );
		$args                   = $query_args;
		$args['fields']         = 'ids';
		$args['posts_per_page'] = -1;
		// Run query
		$loop = new WP_Query( $args );
		foreach ( $loop->posts as $product_id ) {
			if ( ! $this->is_product_visible( $product_id, $country ) ) {
				$post__not_in[] = $product_id;
			}
		}
		// Set `post__not_in`
		$query_args['post__not_in'] = $post__not_in;
		add_action( 'pre_get_posts', array( $this, 'product_by_country_pre_get_posts' ) );
		return $query_args;
	}

	/**
	 * product_by_country_pre_get_posts.
	 *
	 * @version 1.1.0
	 * @since   1.0.0
	 * @todo    [dev] optimization: save query results
	 * @todo    [dev] optimization: `meta_query`
	 * @todo    [dev] (maybe) check `is_admin` and ajax
	 */
	function product_by_country_pre_get_posts( $query ) {
		if ( is_admin() ) {
			return;
		}
		remove_action( 'pre_get_posts', array( $this, 'product_by_country_pre_get_posts' ) );
		$country        = $this->get_country_by_ip();
		$post__not_in   = $query->get( 'post__not_in' );
		$args           = array(
			'post_type'      => 'product',
			'posts_per_page' => -1,
			'fields'         => 'ids',
			'post__not_in'   => $post__not_in,
		);
		// Run query
		$loop = new WP_Query( $args );
		foreach ( $loop->posts as $product_id ) {
			if ( ! $this->is_product_visible( $product_id, $country ) ) {
				$post__not_in[] = $product_id;
			}
		}
		// Set `post__not_in`
		$query->set( 'post__not_in', $post__not_in );
		add_action( 'pre_get_posts', array( $this, 'product_by_country_pre_get_posts' ) );
	}

	/**
	 * product_by_country_purchasable.
	 *
	 * @version 1.1.0
	 * @since   1.0.0
	 */
	function product_by_country_purchasable( $purchasable, $_product ) {
		return ( ! $this->is_product_visible( $this->get_product_id_or_variation_parent_id( $_product ), $this->get_country_by_ip() ) ? false : $purchasable );
	}

	/**
	 * product_by_country_visibility.
	 *
	 * @version 1.1.0
	 * @since   1.0.0
	 */
	function product_by_country_visibility( $visible, $product_id ) {
		return ( ! $this->is_product_visible( $product_id, $this->get_country_by_ip() ) ? false : $visible );
	}

	/**
	 * get_country_by_ip.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @todo    [dev] optimize by saving the result in `$this->saved_country_by_ip`
	 */
	function get_country_by_ip() {
		// Get the country by IP
		$location = ( class_exists( 'WC_Geolocation' ) ? WC_Geolocation::geolocate_ip() : array( 'country' => '' ) );
		// Base fallback
		if ( empty( $location['country'] ) ) {
			$location = wc_format_country_state_string( apply_filters( 'woocommerce_customer_default_location', get_option( 'woocommerce_default_country' ) ) );
		}
		return ( isset( $location['country'] ) ? $location['country'] : '' );
	}

	/**
	 * get_product_id_or_variation_parent_id.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function get_product_id_or_variation_parent_id( $_product ) {
		if ( ! $_product || ! is_object( $_product ) ) {
			return 0;
		}
		if ( $this->is_wc_version_below_3 ) {
			return $_product->id;
		} else {
			return ( $_product->is_type( 'variation' ) ) ? $_product->get_parent_id() : $_product->get_id();
		}
	}

}

endif;

return new Alg_WC_PVBC_Core();
